import os

from planqk.commons.file import write_str_to_file


def write_to_output_directory(file_name: str, content: str,
                              output_directory: str = os.environ.get("OUTPUT_DIRECTORY", "/var/runtime/output")):
    write_str_to_file(output_directory, file_name, content)



import json
import typing
from inspect import Signature
from typing import Any, Dict, get_origin

from pydantic import BaseModel


def files_to_parameters(input_files: Dict[str, str], signature: Signature) -> Dict[str, Any]:
    parameters = {}

    for parameter in signature.parameters.values():
        parameter_name = parameter.name
        parameter_type = parameter.annotation

        # skip parameters without input files
        if parameter_name not in input_files:
            continue

        origin = get_origin(parameter_type)
        if origin:
            parameter_type = origin

        with open(input_files[parameter_name], "r") as file:
            file_content = file.read()

        parameter_value = str_to_parameter_type(file_content, parameter_type)

        parameters[parameter_name] = parameter_value

    return parameters


def str_to_parameter_type(data: str, parameter_type: Any) -> Any:
    if issubclass(parameter_type, str):
        return data

    if issubclass(parameter_type, bool):
        return bool(data)

    if issubclass(parameter_type, int):
        return int(data)

    if issubclass(parameter_type, float):
        return float(data)

    if issubclass(parameter_type, (list, dict)):
        return json.loads(data)

    if issubclass(parameter_type, BaseModel):
        return parameter_type.model_validate_json(data)

    raise ValueError(f"Type {parameter_type} is not supported")


def is_simple_type(value: Any) -> bool:
    return isinstance(value, (str, int, float, bool))


def is_container_type(value: Any) -> bool:
    return value is list or value is tuple or value is typing.Union or value is typing.Optional



from inspect import Signature
from typing import Any, Dict, Tuple, get_origin, get_args

from pydantic import BaseModel

from planqk.commons.parmeters import is_container_type


def generate_parameter_schema(signature: Signature) -> Tuple[Dict[str, Any], Dict[str, Any]]:
    parameter_schemas = {}
    schema_definitions = {}

    # generate schema for each parameter
    parameters = signature.parameters
    for parameter in parameters.values():
        parameter_type = parameter.annotation
        args = get_args(parameter_type)
        origin = get_origin(parameter_type)
        if origin:
            parameter_type = origin

        if len(args) > 0 and is_container_type(origin):
            # nested native lists are not supported
            # it only considers the first type of the given tuple definition
            item_type = args[0]
            if issubclass(item_type, BaseModel):
                schema, schema_definition = generate_pydantic_schema(item_type)
                parameter_schemas[parameter.name] = {"type": "array", "items": schema}
                if schema_definition is not None:
                    schema_definitions.update(schema_definition)
            else:
                parameter_schemas[parameter.name] = {"type": "array"}
        elif issubclass(parameter_type, BaseModel):
            schema, schema_definition = generate_pydantic_schema(parameter_type)
            parameter_schemas[parameter.name] = schema
            if schema_definition is not None:
                schema_definitions.update(schema_definition)
        elif issubclass(parameter_type, list) or issubclass(parameter_type, tuple):
            parameter_schemas[parameter.name] = {"type": "array"}
        elif issubclass(parameter_type, str):
            parameter_schemas[parameter.name] = {"type": "string"}
        # bool needs to be checked before int as otherwise it would be classified as int
        elif issubclass(parameter_type, bool):
            parameter_schemas[parameter.name] = {"type": "boolean"}
        elif issubclass(parameter_type, int):
            parameter_schemas[parameter.name] = {"type": "integer"}
        elif issubclass(parameter_type, float):
            parameter_schemas[parameter.name] = {"type": "number"}
        else:
            # for the rest we assume dict
            parameter_schemas[parameter.name] = {"type": "object", "additionalProperties": {"type": "string"}}

    return parameter_schemas, schema_definitions


def generate_return_schema(signature: Signature) -> Tuple[Dict[str, Any], Dict[str, Any]]:
    return_schema = {}
    schema_definitions = {}

    # generate schema for the return type
    return_type = signature.return_annotation

    args = get_args(return_type)
    origin = get_origin(return_type)
    if origin:
        return_type = origin

    if len(args) > 0 and is_container_type(origin):
        # nested native lists are not supported
        # it only considers the first type of the given tuple definition
        return_type = args[0]

    if issubclass(return_type, BaseModel):
        schema, schema_definition = generate_pydantic_schema(return_type)
        return_schema = schema
        if schema_definition is not None:
            schema_definitions.update(schema_definition)

    return return_schema, schema_definitions


def generate_pydantic_schema(parameter_type) -> Tuple[Dict[str, Any], Dict[str, Any]]:
    if not issubclass(parameter_type, BaseModel):
        raise ValueError("Only Pydantic models are supported")

    schema_definition = None
    schema = parameter_type.model_json_schema(
        ref_template="#/components/schemas/{model}",
        mode="serialization",
    )

    if "$defs" in schema:
        schema_definition = schema.pop("$defs")

    return schema, schema_definition



import inspect
from typing import Any, Dict

from pydantic import BaseModel


def dict_to_pydantic(payload: Dict[str, Any], function_signature: inspect.Signature) -> Any:
    parameter = next(iter(function_signature.parameters.values()))
    parameter_type = parameter.annotation

    if issubclass(parameter_type, BaseModel):
        return parameter_type.model_validate(payload)

    raise ValueError(f"Unsupported data model type: {parameter_type}")



import importlib
import inspect
from typing import Callable


def resolve_function(entrypoint: str) -> Callable:
    module_path, func_name = entrypoint.split(':')

    # import the module dynamically
    module = importlib.import_module(module_path)

    func = getattr(module, func_name)

    return func


def resolve_signature(entrypoint: str) -> inspect.Signature:
    func = resolve_function(entrypoint)

    signature = inspect.signature(func)

    return signature



from typing import Any, Dict

from loguru import logger

from planqk.commons.reflection import resolve_function, resolve_signature


def run_entrypoint(entrypoint: str, entrypoint_arguments: Dict[str, Any]) -> Any:
    signature = resolve_signature(entrypoint)

    logger.debug(f"Parameters in signature: {list(signature.parameters.keys())}")
    logger.debug(f"Found input parameters: {list(entrypoint_arguments.keys())}")

    # remove keys from 'entrypoint_arguments' that are not parameters in the signature
    entrypoint_arguments = {k: v for k, v in entrypoint_arguments.items() if k in signature.parameters}

    func = resolve_function(entrypoint)
    return func(**entrypoint_arguments)



import os
from typing import Dict, Any


def list_directory_files(directory_path: str) -> Dict[str, str]:
    if not os.path.isdir(directory_path):
        raise ValueError(f"Path \"{directory_path}\" must be a directory")

    try:
        files = {}
        for f in os.listdir(directory_path):
            file_path = os.path.join(directory_path, f)
            if os.path.isfile(file_path):
                absolute_path = os.path.abspath(file_path)
                # remove file extension from file name
                f = f.split(".")[0]
                files[f] = absolute_path
        return files
    except FileNotFoundError:
        return {}


def write_str_to_file(directory_path: str, file_name: str, content: str) -> str:
    if not os.path.isdir(directory_path):
        raise ValueError(f"Path \"{directory_path}\" must be a directory")

    file_path = os.path.join(directory_path, file_name)
    with open(file_path, "w", encoding="utf-8") as file:
        file.write(content)

    return file_path


def write_blob_to_file(directory_path: str, file_name: str, content: Any) -> str:
    if not os.path.isdir(directory_path):
        raise ValueError(f"Path \"{directory_path}\" must be a directory")

    file_path = os.path.join(directory_path, file_name)
    with open(file_path, "wb") as file:
        file.write(content)

    return file_path



import os
from typing import Dict, Any

from planqk.commons.openapi import resolve_package_path
from planqk.commons.openapi.file import read_to_dict


def get_template_managed_service() -> Dict[str, Any]:
    package_path = resolve_package_path()
    file_path = os.path.join(package_path, "template-managed-service.yaml")

    openapi = read_to_dict(file_path)

    return openapi
